#!/usr/bin/env node

var prompt = require('prompt')
var program = require('commander')
var package = require('./package.json')
var colors = require("colors/safe")
var path = require('path')
require('shelljs/global')
var appHubAppKey = '2a1fd170-45cd-11e7-b253-3be803aed8f3'
var request = require('request')
var fs = require('fs')
var defaultOutputPath = 'app/build/outputs/apk'
var apkName = 'app-release-unsigned.apk'
var outBaseUrl = 'http://test-xwjr.oss-cn-beijing.aliyuncs.com/apphub/'
var QRCode = require('qrcode')
var configPath = 'pigeon.config.json'
var version = require('./package.json').version

var defaultApkPath = ''




prompt.message = colors.green("Question!");
prompt.delimiter = colors.green("><");


if (!which('git')) {
    echo('Sorry, this script requires git')
    exit(1);
}




program
    .version(version)
    .description('初始化配置文件')
    .command('init')
    .alias('i')
    .action(function() {
        prompt.start();
        prompt.get({
            properties: {
                appName: {
                    description: colors.magenta(`请输入APK名称如（希望金融）?`),
                    required: true
                },
                apkName: {
                    description: colors.magenta(`请输入APK名称如（${apkName}）?`)
                },
                outputPath: {
                    description: colors.magenta(`请输入APK输出路径如（${defaultOutputPath}）?`)
                }
            }
        }, function(err, result) {

            if (!err) {
                fs.writeFile(configPath, JSON.stringify(result), function(err) {
                    if (!err) {
                        console.log('\n')
                        console.log(colors.green('项目初始化成功'))
                        console.log('\n')
                    }
                })
            }
        })
    })

program
    .description('展示配置文件')
    .command('config')
    .alias('c')
    .action(function() {
        fs.readFile(configPath, 'utf8', function(err, data) {
            console.log(colors.green(JSON.stringify(JSON.parse(data))))
        })
    })

program
    .description('构建项目')
    .command('build [platform]')
    .alias('b')
    .action(function(platform) {
        if (!fs.existsSync(configPath)) {
            console.log(colors.red('请先运行 pigeon init 命令初始化配置'))
            return
        }

        platform = platform || 'android'
        console.log(colors.green('项目准备开始构建...'))
        exec('./gradlew assembleRelease')
        prompt.start()
        prompt.get({
            properties: {
                name: {
                    description: colors.magenta("是否上传app 输入y上传，其他键退出?")
                },
                notes: {
                    description: colors.magenta("更新内容"),
                    required: true
                }
            }
        }, function(err, result) {
            if (result.name === 'y') {
                uploadApp(result.notes)
            } else {
                return
            }
        })
    })


program
    .description('发送推送消息')
    .command('notify')
    .alias('n')
    .action(function() {
        if (!fs.existsSync(configPath)) {
            console.log(colors.red('请先运行 pigeon init 命令初始化配置'))
            return
        }
        console.log(colors.green('项目准备开始上传...'))
        prompt.start()
        prompt.get({
            properties: {
                notes: {
                    description: colors.magenta("更新内容"),
                    required: true
                }
            }
        }, function(err, result) {
            uploadApp(result.notes)
        })
    })


function uploadApp(notes) {
    var config = require(path.join(pwd().stdout, configPath))
    var apkPath = path.join(path.join(pwd().stdout, config.outputPath || defaultOutputPath), config.apkName || apkName)
    upload(apkPath, function(err, body) {
        if (!err) {
            QRCode.toFile('qrcode.png', outBaseUrl + '' + body.data.path, function(err, url) {
                upload('qrcode.png', function(err, body) {
                    let imgPath = outBaseUrl + '' + body.data.path
                    rm('./qrcode.png')
                    pushMessage({
                        text: '收到APP更新通知',
                        channel: '#所有人',
                        attachments: [{
                            "title": config.appName,
                            "description": notes,
                            "url": imgPath,
                            "color": "info"
                        }]
                    }, function(err, body) {
                        if (!err) {
                            console.log(colors.green('APP推送成功!'))
                        }
                    })
                })
            })
        }
    })
}



function upload(filePath, callback) {
    var formData = {
        file: fs.createReadStream(filePath),
    };
    request.post({
        url: 'http://cg.kfxfd.cn:9081/apphub/upload',
        headers: {
            'Content-Type': 'application/json; charset=utf-8'
        },
        formData: formData,
        json: true
    }, function optionalCallback(err, httpResponse, body) {
        callback(err, body)
    });
}

function pushMessage(data, callback) {
    var formData = data;
    request.post({
        url: 'https://hooks.pubu.im/services/x14p8db88e2lhn7',
        headers: {
            'Content-Type': 'application/json; charset=utf-8'
        },
        form: formData,
        json: true
    }, function optionalCallback(err, httpResponse, body) {
        callback(err, body)
    })
}


if (!process.argv.slice(2).length) {
    program.outputHelp(make_red);
}

function make_red(txt) {
    return colors.red(txt);
}

program.parse(process.argv)